
use POSIX qw(strftime);
use File::stat;

my $filename;
my $last_mtime;
my %words_lookup;

my $debug;
my $logfile;

sub dolog {
    my ($str) = @_;
    return unless $debug;
    my $ts = strftime("%Y-%m-%d %H:%M:%S", localtime);
    open(OUT, ">>$logfile");
    print OUT "$ts $str\n";
    close(OUT);
}

sub refresh_words {
    my ($id) = @_;

    my $stat = stat($filename);
    # If the filtered words files doesn't exist, nothing to do
    return unless $state;

    # If the filtered words file hasn't been modified, nothing to do
    $last_mtime ||= 0;
    return unless ($stat->mtime > $last_mtime);

    # The filtered words file has changed. Create a new lookup hash
    %words_lookup = ();
    if (open(my $fh, '<', $filename)) {
        while (my $row = <$fh>) {
            chomp($row);
            next if length($row) < 1;
            push @words, lc($row);
        }
        dolog("[$id] - '" . $filename . "' modified. Loaded " . scalar(@words) . " words.");
        close($fh);
    }

    # Create new hash for O(1) lookups
    %words_lookup = map { $_ => 1 } @words;

    # update last modified time
    $last_mtime = $stat->mtime;
}

sub create_config {
    my ($name, $configdata) = @_;
    $filename = $configdata;
    $last_mtime = 0;
    $debug = 1;
    $logfile = "/tmp/wordfilter.log";
}

sub check {
    my ($id, $msg) = @_;

    refresh_words($id);
    my @msg_words = split /\s+/, $msg;
    # check each word in the message to see if has any filter words
    foreach my $msg_word (@msg_words) {
        next unless (exists $words_lookup{lc($msg_word)});
        dolog("[$id] - rejecting message containing '" . $msg_word . "' in message: '" . $msg . "'");
        return 8;
    }
    return 0;
}

sub before_receive {
    my ($request, $response) = @_;
    my $q = ${$request}{qe};
    my $msg = lc($q->{MESSAGE});
    return check($q->{ID}, $msg);
}

